#import "GPGAppKitController.h"

@implementation GPGAppKitController

- (IBAction)signAndEncrypt:(id)sender
{
    //from GPGME
    GPGContext *context = nil;
    GPGRecipients *gpgRecipient = nil;
    GPGKey *gpgSigner = nil;
    GPGData *gpgData = nil;
    GPGData *gpgOutput = nil;

    //from GPGAppKit
    GPGSingleKeySelectionPanel *keySelectionPanel = nil;

    NS_DURING
        //initializers
        context = [[GPGContext alloc] init];
        gpgData = [[GPGData alloc] initWithString: [input stringValue]];
        gpgRecipient = [[GPGRecipients alloc] init];
        keySelectionPanel = [GPGSingleKeySelectionPanel panel];

        //get the recipients

        [keySelectionPanel setMinimumKeyValidity: GPGValidityMarginal];
        [keySelectionPanel setListsSecretKeys: NO];
        [keySelectionPanel setPrompt: @"Choose recipient"];

        [keySelectionPanel runModalForKeyWildcard: nil
                                     usingContext: context];
        
        [gpgRecipient addName:
            [[keySelectionPanel selectedKey] fingerprint]];

        //get the signers

        [keySelectionPanel resetToDefaults];
        [keySelectionPanel setMinimumKeyValidity: GPGValidityUltimate];
        [keySelectionPanel setListsSecretKeys: YES];
        [keySelectionPanel setPrompt: @"Choose signer"];

        [keySelectionPanel runModalForKeyWildcard: nil
                                     usingContext: context];
        
        gpgSigner = [keySelectionPanel selectedKey];

        //set up the context

        [context addSignerKey: gpgSigner];
        [context setPassphraseDelegate: self];
        [context setUsesArmor: YES];

        //do the GPG magic

        gpgOutput = [context encryptedSignedData: gpgData
                                   forRecipients: gpgRecipient
                           allRecipientsAreValid: nil
            ];

        //display output

        [output setStringValue: [gpgOutput string]];

    NS_HANDLER
        NSRunAlertPanel(@"Oops, you broke it.", @"%@",
                        nil, nil, nil, localException);
    NS_ENDHANDLER

    [context release];
    [gpgData release];
    [gpgRecipient release];
}

- (NSString *) context: (GPGContext *)context
      passphraseForKey: (GPGKey *)key
                 again: (BOOL)again
{
    GPGPassphrasePanel *ppanel = [GPGPassphrasePanel panel];

    if (again)
    {
        [ppanel runModalWithPrompt:
            [NSString stringWithFormat: @"Try again: %@, %@",
                [key userID], [key shortKeyID]
                ]
            ];
    }
    else
    {
        [ppanel runModalWithPrompt:
            [NSString stringWithFormat: @"Enter passphrase: %@, %@",
                [key userID], [key shortKeyID]
                ]
            ];
    }

    return [ppanel passphrase];
}

@end
